==> jcquery-5.0.0.tgz: package/$.js <==
/**
 * @template {abstract new (...args: any[]) => any} T
 *
 * @type {{
 *   (type: BigIntConstructor,  arg: any): arg is bigint;
 *   (type: BooleanConstructor, arg: any): arg is boolean;
 *   (type: NumberConstructor,  arg: any): arg is number;
 *   (type: StringConstructor,  arg: any): arg is string;
 *   (type: SymbolConstructor,  arg: any): arg is symbol;
 *   (type: undefined,          arg: any): arg is undefined | null;
 *   <T extends abstract new (...args: any[]) => any>(type: T, arg: any): arg is InstanceType<T>;
 * }}
 *
 * @returns {arg is bigint | boolean | number | string | symbol | undefined | null | InstanceType<T>}
 */
export const $is = (/** @type {T} */ type, /** @type {any} */ arg) => arg?.constructor === type

export const $in = (/** @type {any} */ key, /** @type {any} */ ref) =>
  ($is(String, key) || $is(Number, key) || $is(Symbol, key)) && Object.hasOwnProperty.call(ref ?? Object, key)

export const $eq = /** @returns {boolean} */ (/** @type {any} */ x, /** @type {any} */ y) => {
  if (x === y) {
    return true
  }

  const xConstructor = x?.constructor

  if (xConstructor === y?.constructor) {
    if (xConstructor === Number) {
      return (isNaN(x) && isNaN(y)) || +x === +y
    }

    if (xConstructor === Date) {
      return +x === +y
    }

    if (xConstructor === String || xConstructor === RegExp) {
      return '' + x === '' + y
    }

    if (xConstructor === Array) {
      return x.length === y.length && x.every((/** @type {any} */ item, /** @type {number} */ index) => $eq(item, y[index]))
    }

    if (xConstructor === Object) {
      const keysOfX = Object.keys(x)
      return keysOfX.length === Object.keys(y).length && keysOfX.every(key => $eq(x[key], y[key]))
    }

    if (xConstructor === Set || xConstructor === Map) {
      if (x.size !== y.size) {
        return false
      }
      const xa = [...x]
      const ya = [...y]
      return xa.every(xv => ya.find(yv => $eq(xv, yv)))
    }
  }

  return false
}

/**
 * @typedef {[
 *   string | Node,
 *   ...(Record<string, any> | null | undefined | Node | string | number | $HArgs)[]
 * ]} $HArgs
 */

const /** @type {Record<string, string>} */ NS = {
  xlink: 'http://www.w3.org/1999/xlink'
}

/**
 * @type {{
 *   (namespaceURI?: null | undefined): {
 *     <T extends keyof HTMLElementTagNameMap>(tag:  T, ...args: $HArgs[1][]): HTMLElementTagNameMap[T];
 *     <N extends Node>                       (node: N, ...args: $HArgs[1][]): N;
 *     (...args: $HArgs): Node;
 *   };
 *   (namespaceURI: 'http://www.w3.org/2000/svg'): {
 *     <T extends keyof SVGElementTagNameMap> (tag:  T, ...args: $HArgs[1][]): SVGElementTagNameMap[T];
 *     <N extends Node>                       (node: N, ...args: $HArgs[1][]): N;
 *     (...args: $HArgs): Node;
 *   };
 * }}
 */
const _h = (/** @type {string?=} */ namespaceURI) => {
  const createElement = namespaceURI == null
    ? (/** @type {string } */ tag) => document.createElement(tag)
    : (/** @type {string } */ tag) => document.createElementNS(namespaceURI, tag)

  const h = (/** @type {$HArgs[0]} */ tagOrNode, /** @type {$HArgs[1][]} */ ...args) => {
    const node = $is(String, tagOrNode) ? createElement(tagOrNode) : tagOrNode

    for (const arg of args) {
      let child = null

      if (arg instanceof Node) {
        child = arg
      } else if ($is(String, arg) || $is(Number, arg)) {
        // @ts-expect-error:
        child = new Text(arg)
      } else if ($is(Array, arg)) {
        // @ts-expect-error:
        child = h(...arg)
      } else if (arg != null) {
        for (const name in arg) {
          const value = arg[name]

          if (name[0] === '$') {
            const name1 = name.slice(1)

            if ($is(Object, value)) {
              // @ts-expect-error:
              node[name1] = node[name1] ?? {}
              // @ts-expect-error:
              Object.assign(node[name1], value)
            } else {
              // @ts-expect-error:
              node[name1] = value
            }
          } else if (node instanceof Element) {
            const indexOfColon = name.indexOf(':')

            if (indexOfColon >= 0) {
              const /** @type {string=} */ ns = NS[name.slice(0, indexOfColon)]

              if (ns != null) {
                const basename = name.slice(indexOfColon + 1)

                if (value === true) {
                  node.setAttributeNS(ns, basename, '')
                } else if (value === false) {
                  node.removeAttributeNS(ns, basename)
                } else {
                  node.setAttributeNS(ns, basename, $is(String, value) ? value : '' + value)
                }
              }
            } else {
              if (value === true) {
                node.setAttribute(name, '')
              } else if (value === false) {
                node.removeAttribute(name)
              } else {
                node.setAttribute(name, $is(String, value) ? value : '' + value)
              }
            }
          }
        }
      }

      if (child != null) {
        node.appendChild(child)
      }
    }

    return node
  }

  return h
}

export const $h = _h()

export const $s = _h('http://www.w3.org/2000/svg')

/**
 * @typedef {{
 *   [attributeOrSelector: string]: string | number | $CNode;
 * }} $CNode
 */

const _c = (
  /** @type {$CNode}   */ node,
  /** @type {string}   */ prefix,
  /** @type {string[]} */ result,
  /** @type {(text: string) => string} */ split
) => {
  const /** @type {[$CNode | string[], string][]} */ queue = [[node, prefix]]

  while (queue.length) {
    const [style2, prefix2] = queue.shift() ?? []

    if (style2 == null || prefix2 == null) {
      continue
    }

    if ($is(Array, style2)) {
      result.push(prefix2, prefix2 !== '' ? '{' : '', style2.join(';'), prefix2 !== '' ? '}' : '')
    } else {
      const /** @type {[$CNode | string[], string][]} */ todo = []
      let /** @type {string[]} */ attributes = []
      let attributesPushed = false

      for (const key in style2) {
        const value = style2[key]

        if ($is(String, value) || $is(Number, value)) {
          if (!attributesPushed) {
            attributesPushed = true
            attributes = []
            todo.push([attributes, prefix2])
          }

          attributes.push(`${split(key).replace(/([A-Z])/g, (_, letter) => '-' + letter.toLowerCase())}:${value}`)
        } else {
          attributesPushed = false

          const /** @type {string[]} */ newPrefix = []
          const keySplitted = key.split(',')

          for (const prefixItem of prefix2.split(',')) {
            for (const keyItem of keySplitted) {
              newPrefix.push(prefixItem + keyItem)
            }
          }

          todo.push([value, newPrefix.join(',')])
        }
      }

      queue.unshift(...todo)
    }
  }
}

export const $c = (/** @type {Record<string, $CNode>} */ root, /** @type {string} */ splitter = '$$') => {
  const split = (/** @type {string} */ text) => text.split(splitter)[0]
  const /** @type {string[]} */ result = []

  for (const key in root) {
    const value = root[key]

    if (key[0] === '@') {
      result.push(split(key) + '{')
      _c(value, '', result, split)
      result.push('}')
    } else {
      _c(value, split(key), result, split)
    }
  }

  return result.join('')
}

export const $highlight = (
  /** @type {RegExp} */ pattern,
  /** @type {((chunk: string, index: number) => $HArgs[1])[]} */ highlighters,
  /** @type {string} */ innerText
) => {
  const /** @type {$HArgs[1][]} */ result = []

  innerText.split(pattern).forEach((chunk, index) => {
    index %= highlighters.length
    if (chunk != null) {
      result.push(highlighters[index](chunk, index))
    }
  })

  return result
}

const _locale = (
  /** @type {Record<string, Record<string, string | Record<string, string>>>} */ locales,
  /** @type {string} */ language,
  /** @type {string} */ text,
  /** @type {string} */ version
) => {
  // @ts-expect-error:
  const v = locales?.[language]?.[version]?.[text]

  if ($is(String, v)) {
    return v
  }

  const t = locales?.[language]?.[text]

  return $is(String, t) ? t : text
}

export const $locale = (
  /** @type {Record<string, Record<string, string | Record<string, string>>>} */ locales,
  /** @type {string} */ defaultLanguage,
  /** @type {readonly string[]} */ languages = navigator.languages
) => {
  for (const language of languages) {
    if ($in(language, locales)) {
      // @ts-expect-error:
      return _locale.bind(0, locales, language)
    }
  }

  // @ts-expect-error:
  return _locale.bind(0, locales, defaultLanguage)
}

==> jcquery-5.0.3.tgz: package/$.js <==
/**
 * @template {abstract new (...args: any[]) => any} T
 *
 * @type {{
 *   (type: BigIntConstructor,  arg: any): arg is bigint;
 *   (type: BooleanConstructor, arg: any): arg is boolean;
 *   (type: NumberConstructor,  arg: any): arg is number;
 *   (type: StringConstructor,  arg: any): arg is string;
 *   (type: SymbolConstructor,  arg: any): arg is symbol;
 *   (type: undefined,          arg: any): arg is undefined | null;
 *   <T extends abstract new (...args: any[]) => any>(type: T, arg: any): arg is InstanceType<T>;
 * }}
 *
 * @returns {arg is bigint | boolean | number | string | symbol | undefined | null | InstanceType<T>}
 */
export const $is = (/** @type {T} */ type, /** @type {any} */ arg) => arg?.constructor === type

export const $in = (/** @type {any} */ key, /** @type {any} */ ref) =>
  ($is(String, key) || $is(Number, key) || $is(Symbol, key)) && Object.hasOwnProperty.call(ref ?? Object, key)

export const $eq = /** @returns {boolean} */ (/** @type {any} */ x, /** @type {any} */ y) => {
  if (x === y) {
    return true
  }

  const xConstructor = x?.constructor

  if (xConstructor === y?.constructor) {
    if (xConstructor === Number) {
      return (isNaN(x) && isNaN(y)) || +x === +y
    }

    if (xConstructor === Date) {
      return +x === +y
    }

    if (xConstructor === String || xConstructor === RegExp) {
      return '' + x === '' + y
    }

    if (xConstructor === Array) {
      return x.length === y.length && x.every((/** @type {any} */ item, /** @type {number} */ index) => $eq(item, y[index]))
    }

    if (xConstructor === Object) {
      const keysOfX = Object.keys(x)
      return keysOfX.length === Object.keys(y).length && keysOfX.every(key => $eq(x[key], y[key]))
    }

    if (xConstructor === Set || xConstructor === Map) {
      if (x.size !== y.size) {
        return false
      }
      const xa = [...x]
      const ya = [...y]
      return xa.every(xv => ya.find(yv => $eq(xv, yv)))
    }
  }

  return false
}

/**
 * @typedef {[
 *   string | Node,
 *   ...(Record<string, any> | null | undefined | Node | string | number | $HArgs)[]
 * ]} $HArgs
 */

const /** @type {Record<string, string>} */ NS = {
  xlink: 'http://www.w3.org/1999/xlink'
}

/**
 * @type {{
 *   (namespaceURI?: null | undefined): {
 *     <T extends keyof HTMLElementTagNameMap>(tag:  T, ...args: $HArgs[1][]): HTMLElementTagNameMap[T];
 *     <N extends Node>                       (node: N, ...args: $HArgs[1][]): N;
 *     (...args: $HArgs): Node;
 *   };
 *   (namespaceURI: 'http://www.w3.org/2000/svg'): {
 *     <T extends keyof SVGElementTagNameMap> (tag:  T, ...args: $HArgs[1][]): SVGElementTagNameMap[T];
 *     <N extends Node>                       (node: N, ...args: $HArgs[1][]): N;
 *     (...args: $HArgs): Node;
 *   };
 * }}
 */
const _h = (/** @type {string?=} */ namespaceURI) => {
  const createElement = namespaceURI == null
    ? (/** @type {string } */ tag) => document.createElement(tag)
    : (/** @type {string } */ tag) => document.createElementNS(namespaceURI, tag)

  const h = (/** @type {$HArgs[0]} */ tagOrNode, /** @type {$HArgs[1][]} */ ...args) => {
    const node = $is(String, tagOrNode) ? createElement(tagOrNode) : tagOrNode

    for (const arg of args) {
      let child = null

      if (arg instanceof Node) {
        child = arg
      } else if ($is(String, arg) || $is(Number, arg)) {
        // @ts-expect-error:
        child = new Text(arg)
      } else if ($is(Array, arg)) {
        // @ts-expect-error:
        child = h(...arg)
      } else if (arg != null) {
        for (const name in arg) {
          const value = arg[name]

          if (name[0] === '$') {
            const name1 = name.slice(1)

            if ($is(Object, value)) {
              // @ts-expect-error:
              node[name1] = node[name1] ?? {}
              // @ts-expect-error:
              Object.assign(node[name1], value)
            } else {
              // @ts-expect-error:
              node[name1] = value
            }
          } else if (node instanceof Element) {
            const indexOfColon = name.indexOf(':')

            if (indexOfColon >= 0) {
              const /** @type {string=} */ ns = NS[name.slice(0, indexOfColon)]

              if (ns != null) {
                const basename = name.slice(indexOfColon + 1)

                if (value === true) {
                  node.setAttributeNS(ns, basename, '')
                } else if (value === false) {
                  node.removeAttributeNS(ns, basename)
                } else {
                  node.setAttributeNS(ns, basename, $is(String, value) ? value : '' + value)
                }
              }
            } else {
              if (value === true) {
                node.setAttribute(name, '')
              } else if (value === false) {
                node.removeAttribute(name)
              } else {
                node.setAttribute(name, $is(String, value) ? value : '' + value)
              }
            }
          }
        }
      }

      if (child != null) {
        node.appendChild(child)
      }
    }

    return node
  }

  return h
}

export const $h = _h()

export const $s = _h('http://www.w3.org/2000/svg')

/**
 * @typedef {{
 *   [attributeOrSelector: string]: string | number | $CNode;
 * }} $CNode
 */

const _c = (
  /** @type {$CNode}   */ node,
  /** @type {string}   */ prefix,
  /** @type {string[]} */ result,
  /** @type {(text: string) => string} */ split
) => {
  const /** @type {[$CNode | string[], string][]} */ queue = [[node, prefix]]

  while (queue.length) {
    const [style2, prefix2] = queue.shift() ?? []

    if (style2 == null || prefix2 == null) {
      continue
    }

    if ($is(Array, style2)) {
      result.push(prefix2, prefix2 !== '' ? '{' : '', style2.join(';'), prefix2 !== '' ? '}' : '')
    } else {
      const /** @type {[$CNode | string[], string][]} */ todo = []
      let /** @type {string[]} */ attributes = []
      let attributesPushed = false

      for (const key in style2) {
        const value = style2[key]

        if ($is(String, value) || $is(Number, value)) {
          if (!attributesPushed) {
            attributesPushed = true
            attributes = []
            todo.push([attributes, prefix2])
          }

          attributes.push(`${split(key).replace(/([A-Z])/g, (_, letter) => '-' + letter.toLowerCase())}:${value}`)
        } else {
          attributesPushed = false

          const /** @type {string[]} */ newPrefix = []
          const keySplitted = key.split(',')

          for (const prefixItem of prefix2.split(',')) {
            for (const keyItem of keySplitted) {
              newPrefix.push(prefixItem + keyItem)
            }
          }

          todo.push([value, newPrefix.join(',')])
        }
      }

      queue.unshift(...todo)
    }
  }
}

export const $c = (
  /** @type {Record<string, $CNode>} */ root,
  /** @type {string} */ splitter = '$$'
) => {
  const split = (/** @type {string} */ text) => text.split(splitter)[0]
  const /** @type {string[]} */ result = []

  for (const key in root) {
    const value = root[key]

    if (key[0] === '@') {
      result.push(split(key) + '{')
      _c(value, '', result, split)
      result.push('}')
    } else {
      _c(value, split(key), result, split)
    }
  }

  return result.join('')
}

export const $highlight = (
  /** @type {RegExp} */ pattern,
  /** @type {((chunk: string, index: number) => $HArgs[1])[]} */ highlighters,
  /** @type {string} */ codeJs
) => {
  const /** @type {$HArgs[1][]} */ result = []

  codeJs.split(pattern).forEach((chunk, index) => {
    index %= highlighters.length
    if (chunk != null) {
      result.push(highlighters[index](chunk, index))
    }
  })

  return result
}

const _locale = (
  /** @type {Record<string, Record<string, string | Record<string, string>>>} */ locales,
  /** @type {string} */ language,
  /** @type {string} */ text,
  /** @type {string} */ version
) => {
  // @ts-expect-error:
  const v = locales?.[language]?.[version]?.[text]

  if ($is(String, v)) {
    return v
  }

  const t = locales?.[language]?.[text]

  return $is(String, t) ? t : text
}

export const $locale = (
  /** @type {Record<string, Record<string, string | Record<string, string>>>} */ locales,
  /** @type {string} */ defaultLanguage,
  /** @type {readonly string[]} */ languages = navigator.languages
) => {
  for (const language of languages) {
    if ($in(language, locales)) {
      // @ts-expect-error:
      return _locale.bind(0, locales, language)
    }
  }

  // @ts-expect-error:
  return _locale.bind(0, locales, defaultLanguage)
}
